#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e5|e6|e7|i1> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=zmq
origpkgver=v4_1_5
pkgver=${origpkgver}
ssibuildshims_version=v0_10_00
cmake_version=v3_5_1
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=download.openpkg.org/components/cache/zmq/zeromq-4.1.5.tar.gz

get_this_dir

get_ssibuildshims

set_qualifiers
 
get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------
extra_command=""
case ${extraqual} in
debug) extra_command="--enable-debug";;
prof)  export CXXFLAGS="-g -O2";;
opt)   export CXXFLAGS=-O2;;
* )
   echo "ERROR: please specify debug, opt, or prof"
   usage
   exit 1
;;
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || die "ERROR: fake setup failed"

if [[ "${basequal}" == e[45] ]]
then
  #cxxflg="-std=c++11 -Wno-deprecated-declarations"
  cxxflg="-std=c++11"
#extra_command="${extra_command} -DCLHEP_BUILD_CXXSTD=${cxxflg}"
elif [[ "${basequal}" == e[67] ]]
then
  #cxxflg="-std=c++1y -Wno-deprecated-declarations"
  cxxflg="-std=c++1y"
#extra_command="${extra_command} -DCLHEP_BUILD_CXXSTD=${cxxflg}"
elif [ "${basequal}" = "e10" ]
then
  #cxxflg="-std=c++1y -Wno-deprecated-declarations"
  cxxflg="-std=c++14"
#extra_command="${extra_command} -DCLHEP_BUILD_CXXSTD=${cxxflg}"
elif [ "${basequal}" = "i1" ]
then
  cxxflg="-std=c++11"
#extra_command="${extra_command} -DCLHEP_BUILD_CXXSTD=${cxxflg}"
else
  die "Qualifier $basequal not recognized."
fi

export CXXFLAGS="$cxxflg $CXXFLAGS"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x
cd ${sourcedir}
# Download
tar xf ${tardir}/${pkgtarfile}

pushd zeromq-4.1.5
	
./configure ${extra_command} --prefix=${pkgdir}

(( $? == 0 )) || die "ERROR: configure failed."
make clean || die "ERROR: make clean failed."
make -j ${ncore} || die "ERROR: make failed."
make install  || die "ERROR: make install failed."

popd

set +x

#get cppzmq headers
pushd ${pkgdir}
curl -LO https://github.com/zeromq/cppzmq/archive/master.tar.gz
tar xfzv master.tar.gz
cp cppzmq-master/*hpp include
popd

pushd ${pkgdir}/lib
mkdir -p zmq/cmake

cat > zmq/cmake/zmq-config.cmake <<EOF
set( zmq_UPS_VERSION v${pkgdotver} )

find_library(ZMQ NAMES zmq PATHS \$ENV{ZMQ_LIB_DIR} NO_DEFAULT_PATH )
set( ZMQLIBS \${ZMQ} )
EOF

cat > zmq/cmake/zmq-config-version.cmake <<EOF
set( PACKAGE_VERSION "4.1.5" )
if( "\${PACKAGE_FIND_VERSION}" VERSION_EQUAL "4.1.5")
  set(PACKAGE_VERSION_EXACT 1)
endif()
if( "\${PACKAGE_FIND_VERSION_MAJOR}.\${PACKAGE_FIND_VERSION_MINOR}" EQUAL "4.1" )
  set(PACKAGE_VERSION_COMPATIBLE 4)
elseif( "\${PACKAGE_FIND_VERSION_MAJOR}" EQUAL "4" )
  # for now backward compatible if minor version is less
  if( \${PACKAGE_FIND_VERSION_MINOR}  LESS 4 )
    set(PACKAGE_VERSION_COMPATIBLE 4)
  endif()
endif()
set( ZMQ_STATIC_LIB "OFF")
EOF

popd

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "zmq is installed at ${ZMQ_BASE_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
